/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Drag de Go code.
 *
 * The Initial Developer of the Original Code is
 * Yukichi Kamata.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
var ddgUtils = {
	Cc : Components.classes,
	Log : function(msg){
		var Console = this.Cc['@mozilla.org/consoleservice;1']
							.getService(nsCI.nsIConsoleService);
		Console.logStringMessage('Drag de Go: \n' + msg);
	},
	
	MostRecentWindow : function(winType){
		var WindowMediator = this.Cc['@mozilla.org/appshell/window-mediator;1'].
							getService(nsCI.nsIWindowMediator);
		return WindowMediator.getMostRecentWindow(winType);
	},
	
	sBarMessage : function(msg, timeToClear){
		if (!ddg_StatusBar) return;
		if(msg != "")
			ddg_StatusBar.label = 'Drag de Go:' + msg;
		else
			ddg_StatusBar.label = '';
		if (timeToClear)
			setTimeout(ddgUtils.sBarMessage, timeToClear, "", 0);
	},
	
	getDir : function(dirName){//profile:getDir("ProfD") install directory:getDir("CurProcD")
		return this.Cc["@mozilla.org/file/directory_service;1"]
			.getService(nsCI.nsIProperties)
			.get(dirName, nsCI.nsILocalFile);
	},
	
	initFile : function(aTitle, aName){
		var fp = this.Cc["@mozilla.org/filepicker;1"].
					createInstance(nsCI.nsIFilePicker);
		fp.init(window, aTitle, fp.modeSave);
		fp.defaultExtension = "txt";
		fp.defaultString = aName;
		fp.appendFilters(fp.filterText);
		fp.appendFilters(fp.filterAll);
		return fp;
	},
	
	saveText : function(aFile, aText){
		var stream = this.Cc['@mozilla.org/network/file-output-stream;1']
						.createInstance(nsCI.nsIFileOutputStream);
		stream.init(aFile, 0x20|0x02|0x08, 0666, 0);
		stream.write(aText, aText.length);
		stream.close();
	},
	
	initFileWithPath : function(aPath){
		var file = this.Cc["@mozilla.org/file/local;1"].
						createInstance(nsCI.nsILocalFile);
		file.initWithPath(aPath);
		return file;
	},
	
	ConvertFromUnicode : function(aText, aChar){
			var UConv = this.Cc['@mozilla.org/intl/scriptableunicodeconverter'].
							getService(nsCI.nsIScriptableUnicodeConverter);
			UConv.charset = aChar;
			return UConv.ConvertFromUnicode(aText);
	},
	
	setLeafName : function(file){
		var extCount = 0;
		while (file.exists()) {
			extCount++;
			if (extCount == 1) {
				if (file.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i))
					file.leafName = file.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, "(2)$&");
				else
					file.leafName = file.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
			}else {
				file.leafName = file.leafName.replace(/^(.*\()\d+\)/, "$1" + (extCount+1) + ")");
			}
		}
		return file;
	},
	
	getTransferData : function(aContentType, aDragSession){
		var transfer = this.Cc["@mozilla.org/widget/transferable;1"].
				createInstance(nsCI.nsITransferable);
		transfer.addDataFlavor(aContentType);
		aDragSession.getData (transfer, 0);
		var Data = {};
		Data.dataObj = new Object();
		Data.len = new Object();
		try{
			transfer.getTransferData(aContentType, Data.dataObj, Data.len);   
		} catch (ex) {}
		return Data;
	}
}

var ddg_StringS = {
	_SS : null,
	get SS(){
		if(!this._SS)
			this._SS = Components.classes["@mozilla.org/intl/stringbundle;1"].
						getService(Components.interfaces.nsIStringBundleService);
		return this._SS;
	},
	_SB : null,
	get SB(){
		if(!this._SB)
			this._SB = this.SS.createBundle("chrome://dragdego/locale/dragdego.properties");
		return this._SB;
	},
	getByName : function(name){
		return this.SB.GetStringFromName(name);
	}
}

var ddg_Security = {
	checkURI : function(aURI){
		if (/^\s*(javascript|data):/.test(aURI)) return false;
		var uriStr = aURI.replace(/^\s*|\s*$/g, '');
		uriStr = this.fixupSchemer(uriStr);
		if (!(/^(https?):/.test(uriStr))) return uriStr;
		
		var sourceURL = getBrowser().currentURI.spec;
		const nsIScriptSecurityManager = Components.interfaces.nsIScriptSecurityManager;
		var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
						.getService(nsIScriptSecurityManager);
		const nsIScriptSecMan = Components.interfaces.nsIScriptSecurityManager;
		try {
			secMan.checkLoadURIStr(sourceURL, uriStr, nsIScriptSecMan.DISALLOW_SCRIPT_OR_DATA);
		} catch (e) {
			uriStr = false;
		}
		return uriStr;
	},

	fixupSchemer : function(aURI) {
		var table = "ttp=>http,tp=>http,p=>http,ttps=>https,tps=>https,ps=>https";
		var regexp = new RegExp();
		if (aURI.match(regexp.compile(
				'^('+ table.replace(/=>[^,]+|=>[^,]+$/g, '')
						.replace(/\s*,\s*/g, '|')+
					'):',	'g'  )
		)) {
			var target = RegExp.$1;
			table.match(regexp.compile('(,|^)'+target+'=>([^,]+)'));
			aURI = aURI.replace(target, RegExp.$2);
		}
		aURI = aURI.replace(/^([a-z]+[a-z\d]+\.[a-z\d\.]+\/)/i, "http://$1");
		return aURI;
	}
}

function ddg_findBar(aData, mode){
	var findToolbar = document.getElementById("FindToolbar");
	var findField = document.getElementById("find-field");
	var Text = "";
	if(mode == 0 && findToolbar && !findToolbar.hidden){
		findField.value = Text;
		if('gFindBar' in window){ 
			gFindBar.find(Text);
			if(!aData.tipChanged) gFindBar.closeFindBar();
		}else{
			find(Text);
			if(!aData.tipChanged) closeFindBar();
		}
	}else if(mode == 1){
		if(findToolbar.hidden)
			if('gFindBar' in window) gFindBar.openFindBar();
			else openFindBar();
		if(!aData.tipChanged) Text = aData.text;
		findField.value = Text;
		if('gFindBar' in window)
			gFindBar.find(Text);
		else
			find(Text);
	}
}

var ddg_Preferences = {
	ddg_prefRoot : "dragdego.settings.",
	Prefs : [],
	
	/* default values */
	defPrefs : function(){
		this.OpenIn = ["go-f","go-f"];
		this.Taction = [0,"paste-findbar","delete-findbar","paste-searchbar","search-f"];
		this.Laction = [0,"paste-findbar","delete-findbar","saveMenu","go-f"];
		this.Iaction = [1,"save-Img","save-Img","saveMenu","go-f"];
		this.Xaction = [1,"install-addon","install-addon","saveMenu","install-addon"];
		this.Saction = [2, "paste-findbar", "delete-findbar", "paste-findbar", "delete-findbar"];
		this.tipStyle = [20, 9, "#333333", "#FFFFCC", 100,"Comic%20Sans%20MS"];
		this.showTooltip = true;
		this.moveEnable = true;
		this.ConfirmSaving = true;
		this.ShowAddFolderMenu = true;
		this.ShowOptionMenu = true;
		this.ShowSaveAsMenu = true;
		this.enableToChangeUrlGest = true;
		this.EnableTimeout = true;
		this.TimeoutPeriod = 200;
		this.DragContextButton = 2;
		this.DragContextMenus = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1];
		this.ShowProgress = true;
		this.CharSet = "UTF-8";
	},
	
	_pref : null,
	get pref() {
		if (!this._pref)
			this._pref = Components.classes['@mozilla.org/preferences-service;1'].
						getService(nsCI.nsIPrefBranch);
		return this._pref;
	},
	
	loadPrefs : function(){
		this.Prefs = new this.defPrefs();
		with(this.Prefs){
			Taction = this.get('textaction', Taction);
			Laction = this.get('linkaction', Laction);
			Iaction = this.get('imgaction', Iaction);
			Xaction = this.get('xpiaction', Xaction);
			Saction = this.get('sourceaction', Saction);
			OpenIn  = this.get('OpenIn', OpenIn);
			showTooltip = this.get('showTooltip', showTooltip);
			moveEnable = this.get('moveEnable', moveEnable);
			tipStyle = this.get('IndicatorSet', tipStyle);
			tipStyle[0] = parseInt(tipStyle[0]);
			tipStyle[1] = parseInt(tipStyle[1]);
			tipStyle[4] = parseInt(tipStyle[4]);
			ConfirmSaving = this.get('ConfirmSaving', ConfirmSaving);
			ShowAddFolderMenu = this.get('ShowAddFolderMenu', ShowAddFolderMenu);
			ShowOptionMenu = this.get('ShowOptionMenu', ShowOptionMenu);
			ShowSaveAsMenu = this.get('ShowSaveAsMenu', ShowSaveAsMenu);
			enableToChangeUrlGest = this.get('enableToChangeUrlGest', enableToChangeUrlGest);
			EnableTimeout = this.get('EnableTimeout', EnableTimeout);
			TimeoutPeriod = this.get('TimeoutPeriod', TimeoutPeriod);
			DragContextButton = this.get('DragContextButton', DragContextButton);
			DragContextMenus = this.get('DragContextMenus', DragContextMenus);
			ShowProgress = this.get('ShowProgress', ShowProgress);
			CharSet = this.get('CharSet', CharSet);
		}
	},
	
	savePrefs : function(){
		with(this.Prefs){
			this.set('textaction', Taction);
			this.set('linkaction', Laction);
			this.set('imgaction', Iaction);
			this.set('xpiaction', Xaction);
			this.set('sourceaction', Saction);
			this.set('OpenIn', OpenIn);
			this.set('showTooltip', showTooltip);
			this.set('moveEnable', moveEnable);
			this.set('IndicatorSet', tipStyle);
			this.set('ConfirmSaving', ConfirmSaving);
			this.set('ShowAddFolderMenu', ShowAddFolderMenu);
			this.set('ShowOptionMenu', ShowOptionMenu);
			this.set('ShowSaveAsMenu', ShowSaveAsMenu);
			this.set('enableToChangeUrlGest', enableToChangeUrlGest);
			this.set('EnableTimeout', EnableTimeout);
			this.set('TimeoutPeriod', TimeoutPeriod);
			this.set('DragContextButton', DragContextButton);
			this.set('DragContextMenus', DragContextMenus);
			this.set('ShowProgress', ShowProgress);
			this.set('CharSet', CharSet);
		}
	},
	
	get : function (prefname, value) {
		prefname = this.ddg_prefRoot + prefname;
		try {
			if (typeof(value) == "boolean")
				var scelta = this.pref.getBoolPref(prefname);
			else if (value instanceof Array)
				var scelta = this.pref.getCharPref(prefname).split(" ");
			else if (typeof(value) == "string")
				var scelta = this.pref.getCharPref(prefname);
			else if (typeof(value) == "number")
				var scelta = this.pref.getIntPref(prefname);
			return scelta;
		} catch (e) {
			this.set(prefname, value);
			return value;
		}
	},
	set : function(prefname, value) {
		if(!/^dragdego.settings./.test(prefname)) 
			prefname = this.ddg_prefRoot + prefname;
		
		if (typeof(value) == "boolean")
			this.pref.setBoolPref(prefname, value);
		else if (value instanceof Array)
			this.pref.setCharPref(prefname, value.join(" "));
		else if (typeof(value) == "string")
			this.pref.setCharPref(prefname, value);
		else if (typeof(value) == "number")
			this.pref.setIntPref(prefname, value);
	},
	removeBranch: function() {
		try {
			this.pref.deleteBranch(this.ddg_prefRoot);
		} catch(e) {
			ddgUtils.Log("Error in ddg_Preferences\nFails to delete prefBranch.");
		}
	}
}

var ddg_folderManager = {
	Cc : Components.classes,
	RDFfoldersRoot : "urn:dragdego:folders:",
	NC_TM : [],
	foldersType : [],
	RDFService: null,
	CONUtils: null,
	DATASource: null,
	IOService: null,
	NC_NS : "http://home.netscape.com/NC-rdf#",
	
	// call by TM_init
	init: function(target) {
		this._init();
		var items = new Array();
		var path = this.foldersType[target];
		var folders = this.initContainer(path).GetElements();
		//
		while (folders.hasMoreElements()) {
			var folder = folders.getNext();
			var name = this.getLiteralValue(folder, "name");
			var path = this.getLiteralValue(folder, "path");
			if(name && path){
				items.push(new this.createItem(name, path, folder));
			}
		}
		return items;
	},
	
	_init : function(){
		this.initService();
		this.setFolder_Content();
		this.setFolder_Type();
	},
	// init common services
	initService: function() {
		this.RDFService = this.Cc["@mozilla.org/rdf/rdf-service;1"]
						.getService(nsCI.nsIRDFService);
		this.CONUtils = this.Cc["@mozilla.org/rdf/container-utils;1"]
						.getService(nsCI.nsIRDFContainerUtils);
		//init datasource
		var file = ddgUtils.getDir("ProfD");
		file.append("dragdego.rdf");
		this.IOService = this.Cc["@mozilla.org/network/io-service;1"]
						.getService(nsCI.nsIIOService);
		var uri = this.IOService.newFileURI(file).spec;
		try {
			this.DATASource = this.RDFService.GetDataSourceBlocking(uri);
		} catch (e) {
			ddgUtils.Log("Error in initService" + "\n" + "caller " + this.initService.caller.toString().split(";")[0]);
		}
	},
	
	getDataSource:function(){
		if (!this.DATASource) this.initService();
		return this.DATASource;
	},
	
	getFolders:function(target){
		if (!this.DATASource) this.initService();
		this.setFolder_Content();
		this.setFolder_Type();
		var items = new Array();
		var path = this.foldersType[target];
		var folders = this.initContainer(path).GetElements();
		while(folders.hasMoreElements()){
			var item = [];
			var folder = folders.getNext();
			item.name = this.getLiteralValue(folder, "name");
			item.path = this.getLiteralValue(folder, "path");
			items.push(item);
		}
		return items;
	},
	
	getDfFolder:function(target){
		if (!this.DATASource) this.initService();
		this.setFolder_Content();
		this.setFolder_Type();
		//var items = new Array();
		var path = this.foldersType[target];
		var folders = this.initContainer(path).GetElements();
		if(folders.hasMoreElements()){
			var folder = folders.getNext();
			var path = this.getLiteralValue(folder, "path");
			if(path) return path;
		}
		return false;
	},
	
	getValue: function(node, label, typeID, def) {
		if (typeof(node) == "string") node = this.RDFService.GetResource(node);
		label = this.NC_TM[label];
		var rdfNode = this.DATASource.GetTarget(node, label, true);
		return (rdfNode instanceof nsCI[typeID]) ? rdfNode.Value : def;
	},
	
	getLiteralValue: function(node, arc, def) {
		if (typeof(def) == "undefined") def = "";
		return this.getValue(node, arc, "nsIRDFLiteral", def);
	},
	
	setFolder_Content: function() {
		var rdfLabels = ["name","path"];
		for each(var Label in rdfLabels) {
			this.NC_TM[Label] = this.RDFService.GetResource(this.NC_NS + Label);
		}
	},
	
	setFolder_Type: function() {
		var Folders = ["text","images","links","xpis","externalapp"];
		for each(var Folder in Folders) {
			this.foldersType[Folder] = this.RDFfoldersRoot + Folder;
		}
	},
	
	setLiteral: function(node, arc, value) {
		if (typeof(node) == "string") node = this.RDFService.GetResource(node);
		arc = this.NC_TM[arc];
		value = this.RDFService.GetLiteral(value);
		this.changeValue(node, arc, value);
	},
	
	changeValue: function(node, arc, newValue) {
		if (this.DATASource.hasArcOut(node, arc)) {
			var oldValue = this.DATASource.GetTarget(node, arc, true);
			if (newValue != oldValue) this.DATASource.Change(node, arc, oldValue, newValue);
		} else this.DATASource.Assert(node, arc, newValue, true);
	},
	
	newNode: function(folderType) {
		if(!this.DATASource) this._init();
		var item = new this.createItem();
		window.openDialog("chrome://dragdego/content/prefs/setfolder.xul",
							"_blank", "chrome,centerscreen,modal,resizable=yes,dependent=yes", item, folderType);
		
		if (item.path == "" || item.name == "") return false;
		
		var folders = this.initContainer(this.foldersType[folderType]);
		var Entry = this.RDFService.GetAnonymousResource();
		this.setLiteral(Entry, "name", item.name);
		this.setLiteral(Entry, "path", item.path);
		item.node = Entry;
		folders.AppendElement(Entry);
		this.dataFlush();
		return item;
	},
	
	editNode: function(item) {
		this.setLiteral(item.node, "name", item.name);
		this.setLiteral(item.node, "path", item.path);
		this.dataFlush();
	},
	
	moveNode: function(rdfNode, index, folderType) {
		var folders = this.initContainer(this.foldersType[folderType]);
		folders.RemoveElement(rdfNode, true);
		folders.InsertElementAt(rdfNode, index, true);
		this.dataFlush();
	},
	
	deleteNode: function(rdfNode, folderType) {
		var arcOut = this.DATASource.ArcLabelsOut(rdfNode);
		while (arcOut.hasMoreElements()) {
			var aLabel = arcOut.getNext();
			if (aLabel instanceof nsCI.nsIRDFResource) {
				var aTarget = this.DATASource.GetTarget(rdfNode, aLabel, true);
				this.DATASource.Unassert(rdfNode, aLabel, aTarget);
			}
		}
		var folders = this.initContainer(this.foldersType[folderType]);
		folders.RemoveElement(rdfNode, true);
		this.dataFlush();
	},
	
	dataFlush: function() {
		this.DATASource.QueryInterface(nsCI.nsIRDFRemoteDataSource).Flush();
	},
	
	initContainer: function(node) {
		var pNode = node;
		if (typeof(node) == "string") node = this.RDFService.GetResource(node);
		try{
			return this.CONUtils.MakeSeq(this.DATASource, node);
		} catch (e) {
			ddgUtils.Log("Error in initContainer" + "\n" + "node = " + pNode + "\n\n" + "caller " + this.initContainer.caller.toString().split(";")[0]);
			return "error";
		}
	},
	
	addNewFolder : function(aData){
		this.newNode(aData.target);
	},
	
	createItem : function(name, path, node){
		this.name = name ? name : ""
		this.path = path ? path : "";
		this.node = node ? node : "";
	}
}
